#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h> 

int init_socket(char *host);
void *recv_thread(void *argv);
int input_cmd();

char login_id[13];
char login_pw[13];

int client;
int req_id;

char jobs_message[1024];

int main(int argc, char *argv[])
{
    if(argc < 2) {
        printf("%s [host]\n", argv[0]);
        return 0;
    }
    
    pthread_t thread;
    
    req_id = 0;
    init_socket(argv[1]);
    jobs_message[0] = 0;
    
    pthread_create(&thread, NULL, &recv_thread, (void *)NULL);
    
    while(input_cmd());
    
    return 0;
}

int init_socket(char *host) {
    struct hostent *server;
    struct sockaddr_in serveraddr;
    
    client = socket(AF_INET, SOCK_STREAM, 0);
    if (client < 0) {
        printf("socket error!");
        exit(1);
    }
    
    server = gethostbyname(host);
    if (server == NULL) {
        printf("ERROR, no such host as %s\n", host);
        exit(0);
    }

    /* build the server's Internet address */
    bzero((char *) &serveraddr, sizeof(serveraddr));
    serveraddr.sin_family = AF_INET;
    bcopy((char *)server->h_addr, (char *)&serveraddr.sin_addr.s_addr, server->h_length);
    serveraddr.sin_port = htons(6000);

    /* connect: create a connection with the server */
    if (connect(client, &serveraddr, sizeof(serveraddr)) < 0) {
        printf("connect error!");
        exit(1);
    }
}

void *recv_packet(char *buf, int size) {
    char type = buf[0];
    int req_id = ((buf[1] << 24)&0xff000000) | ((buf[2] << 16)&0xff0000) | ((buf[3] << 8)&0xff00) | (buf[4]&0xff);
    char res_state = buf[5];
    int i, length;
    char user_id[13];
    
    if(type != -1) printf("[Response] Req:%d State:%d\n", req_id, res_state);
    
    if(res_state == -1) {
        printf("Auth Fail!\n");
    } else {
        if(type == 4 && res_state == 0) {
            length = ((buf[6] << 24)&0xff000000) | ((buf[7] << 16)&0xff0000) | ((buf[8] << 8)&0xff00) | (buf[9]&0xff);
            
            printf("Id\tIsAdmin\n");
            for(i=0; i<length; i++) {
                memcpy(user_id, buf+10+13*i, 12);
                printf("%s\t%c\n", user_id, buf[10+13*i+12] == 1 ? 'O' : 'X');
            }
            printf("User Count : %d\n", length);
        } else if(type == -1) {
            length = ((buf[6] << 24)&0xff000000) | ((buf[7] << 16)&0xff0000) | ((buf[8] << 8)&0xff00) | (buf[9]&0xff);
            memcpy(jobs_message, buf+10, length * sizeof(char));
            jobs_message[length] = 0;
        }
    }
}

void *recv_thread(void *argv)
{
    char header[5]; // version, length
    int result;
    int size;
    
    while(1) {
        result = recv(client, header, 5, MSG_WAITALL);
        if(result < 1) {
            close(client);
            return;
        }
        
        // 빅 엔디안 사이즈 계산
        size =  ((header[1] << 24)&0xff000000) | ((header[2] << 16)&0xff0000) | ((header[3] << 8)&0xff00) | (header[4]&0xff);

        char * buf = (char *)malloc(sizeof(char) * size);
        result = recv(client, buf, size, MSG_WAITALL);
        if(result < 1) {
            close(client);
            return;
        } else if(size == result) {
            recv_packet(buf, size);
        }
        
        free(buf);
    }
}

void login(char *id, char *pw) {
    memset(login_id, 0, 13);
    memset(login_pw, 0, 13);
    memcpy(login_id, id, strlen(id));
    memcpy(login_pw, pw, strlen(pw));
}

void print(char *path) {
    int p_size;
    long size;
    char *buf;
    int fp = fopen(path, "r");
    if(!fp) {
        printf("Not Exist!\n");
        return;
    }
    
    fseek(fp, 0, SEEK_END); 
    size = ftell(fp);
    fseek(fp, 0, SEEK_SET);
    
    printf("file_size : %d\n", size);
    
    p_size = 1 + 4 + 12 + 12 + 4 + size;
    
    // ver 0
    // length 4
    // type 1
    // req_id 4
    // user_id 12
    // user_pw 12
    // file_size
    // content
    ++req_id;
    
    buf = (char *)malloc((1 + 4 + 1 + 4 + 12 + 12 + 4 + size)*sizeof(char));
    buf[0] = 0;
    buf[1] = (p_size >> 24) & 0xff;
    buf[2] = (p_size >> 16) & 0xff;
    buf[3] = (p_size >> 8) & 0xff;
    buf[4] = p_size & 0xff;
    
    buf[5] = 0; // req_type
    
    buf[6] = (req_id >> 24) & 0xff;
    buf[7] = (req_id >> 16) & 0xff;
    buf[8] = (req_id >> 8) & 0xff;
    buf[9] = req_id & 0xff;
    
    memcpy(buf+10, login_id, 12);
    memcpy(buf+22, login_pw, 12);
    
    buf[34] = (size >> 24) & 0xff;
    buf[35] = (size >> 16) & 0xff;
    buf[36] = (size >> 8) & 0xff;
    buf[37] = size & 0xff;
    fread(buf + 38, 1, size, fp);
    
    printf("size : %d\n", 1 + 4 + 1 + 4 + 12 + 12 + 4 + size);
    
    write(client, buf, 1 + 4 + 1 + 4 + 12 + 12 + 4 + size);

    free(buf);
}

void useradd(char *id, char *pw)
{
    int p_size;
    long size;
    char *buf;
    
    p_size = 1 + 4 + 12 + 12 + 12 + 12;
    
    
    ++req_id;
    
    buf = malloc(1 + 4 + 1 + 4 + 12 + 12 + 12 + 12);
    buf[0] = 0;
    buf[1] = (p_size >> 24) & 0xff;
    buf[2] = (p_size >> 16) & 0xff;
    buf[3] = (p_size >> 8) & 0xff;
    buf[4] = p_size & 0xff;
    buf[5] = 2; // req_type
    buf[6] = (req_id >> 24) & 0xff;
    buf[7] = (req_id >> 16) & 0xff;
    buf[8] = (req_id >> 8) & 0xff;
    buf[9] = req_id & 0xff;
    memcpy(buf+10, login_id, 12);
    memcpy(buf+22, login_pw, 12);
    memcpy(buf+34, id, 12);
    memcpy(buf+46, pw, 12);
    
    write(client, buf, 1 + 4 + 1 + 4 + 12 + 12 + 12 + 12);

    free(buf);
}

void userdel(char *id)
{
    int p_size;
    long size;
    char *buf;
    
    p_size = 1 + 4 + 12 + 12 + 12;
    
    
    ++req_id;
    
    buf = malloc(1 + 4 + 1 + 4 + 12 + 12 + 12);
    buf[0] = 0;
    buf[1] = (p_size >> 24) & 0xff;
    buf[2] = (p_size >> 16) & 0xff;
    buf[3] = (p_size >> 8) & 0xff;
    buf[4] = p_size & 0xff;
    buf[5] = 3; // req_type
    buf[6] = (req_id >> 24) & 0xff;
    buf[7] = (req_id >> 16) & 0xff;
    buf[8] = (req_id >> 8) & 0xff;
    buf[9] = req_id & 0xff;
    memcpy(buf+10, login_id, 12);
    memcpy(buf+22, login_pw, 12);
    memcpy(buf+34, id, 12);
    
    write(client, buf, 1 + 4 + 1 + 4 + 12 + 12 + 12);

    free(buf);
}

void userlist()
{
    int p_size;
    long size;
    char *buf;
    
    p_size = 1 + 4 + 12 + 12;
    
    ++req_id;
    
    buf = malloc(1 + 4 + 1 + 4 + 12 + 12);
    buf[0] = 0;
    buf[1] = (p_size >> 24) & 0xff;
    buf[2] = (p_size >> 16) & 0xff;
    buf[3] = (p_size >> 8) & 0xff;
    buf[4] = p_size & 0xff;
    buf[5] = 4; // req_type
    buf[6] = (req_id >> 24) & 0xff;
    buf[7] = (req_id >> 16) & 0xff;
    buf[8] = (req_id >> 8) & 0xff;
    buf[9] = req_id & 0xff;
    memcpy(buf+10, login_id, 12);
    memcpy(buf+22, login_pw, 12);

    write(client, buf, 1 + 4 + 1 + 4 + 12 + 12);

    free(buf);
}

void jobs()
{
    printf("%s\n", jobs_message);
}

void rmjob(int job_id)
{
    int p_size;
    long size;
    char *buf;
    
    p_size = 1 + 4 + 12 + 12 + 4;
    
    ++req_id;
    
    buf = malloc(1 + 4 + 1 + 4 + 12 + 12 + 4);
    buf[0] = 0;
    buf[1] = (p_size >> 24) & 0xff;
    buf[2] = (p_size >> 16) & 0xff;
    buf[3] = (p_size >> 8) & 0xff;
    buf[4] = p_size & 0xff;
    buf[5] = 1; // req_type
    buf[6] = (req_id >> 24) & 0xff;
    buf[7] = (req_id >> 16) & 0xff;
    buf[8] = (req_id >> 8) & 0xff;
    buf[9] = req_id & 0xff;
    memcpy(buf+10, login_id, 12);
    memcpy(buf+22, login_pw, 12);
    buf[34] = (job_id >> 24) & 0xff;
    buf[35] = (job_id >> 16) & 0xff;
    buf[36] = (job_id >> 8) & 0xff;
    buf[37] = job_id & 0xff;
    
    write(client, buf, 1 + 4 + 1 + 4 + 12 + 12 + 4);

    free(buf);
}


int input_cmd() {
    char buf[1024];
    printf("%d # ", req_id + 1);
    gets(buf);
    
    if(!strncmp(buf, "login", 5)) { // login id pw
        char id[13];
        char pw[13];
        memset(id, 0, 13);
        memset(pw, 0, 13);
        sscanf(buf, "login %s %s", id, pw);
        
        login(id, pw);
    } else if(!strncmp(buf, "print", 5)) { // print a.txt
        char path[255];
        sscanf(buf, "print %s", path);
        print(path);
    } else if(!strncmp(buf, "useradd", 7)) {// useradd asdf asdf
        char id[13];
        char pw[13];
        memset(id, 0, 13);
        memset(pw, 0, 13);
        sscanf(buf, "useradd %s %s", id, pw);
        
        useradd(id, pw);
    } else if(!strncmp(buf, "userdel", 7)) {// userdel asdf
        char id[13];
        memset(id, 0, 13);
        sscanf(buf, "userdel %s", id);
        
        userdel(id);
    } else if(!strncmp(buf, "userlist", 8)) {// userlist
        userlist();
    } else if(!strncmp(buf, "jobs", 4)) {// jobs
        jobs();
    } else if(!strncmp(buf, "rmjob", 5)) {// rmjob
        int job_id;
        sscanf(buf, "rmjob %d", &job_id);
        rmjob(job_id);
    } else if(!strncmp(buf, "help", 4)) {
        printf("login [id] [pw]\n");
        printf("print [path]\n");
        printf("useradd [id] [pw]\n");
        printf("userdel [id]\n");
        printf("userlist\n");
        printf("jobs\n");
        printf("rmjob [job_id]\n");
    } else {
        printf("Error : input # help\n");
    }
    
    return 1;
}