#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include "../../process.h"
#include "../../unittest.h"
#include "process.h"
#include "test.h"
#include "../../databases/ink_paper_state_database/process.h"


static struct RequestRefillInfo get_request_info();

static int server_sock = -1;
static void init_socket();

static fd_set sock_fd_set;
static fd_set sock_read_fds;

static struct timeval timeout;

// 프로세스 정의
struct VirtualRefillInterface virtual_refill_interface = {
    .get_request_info = get_request_info
};


void init_virtual_refill_interface() {
    printf("Initialization Virtual Refill Interface...\t\t");
    set_process(VIRTUAL_REFILL_INTERFACE, &virtual_refill_interface);
	SET_UNIT_TEST(VIRTUAL_REFILL_INTERFACE, virtual_refill_interface_unit_test);
	init_socket();
    printf("\033[92m[OK]\033[0m\n");
}

static struct RequestRefillInfo get_request_info() {
    struct sockaddr_in clientaddr;
    int client_len, len;
    char buf[128];
    
    sock_read_fds = sock_fd_set;

    struct RequestRefillInfo info;
    info.type = -1;
    info.value = -1;
    
    if(select(server_sock+1, &sock_read_fds, 0, 0, &timeout) == -1)
    {
        printf("Server-select() error lol!");
        exit(1);
    }

    if(FD_ISSET(server_sock, &sock_read_fds))
    {
        len = recvfrom(server_sock, buf, 128, 0, (struct sockaddr *)&clientaddr, &client_len);

        if(len > 4) {
            info.type = buf[0];
            info.value = (buf[1] << 24)&0xff000000 | (buf[2] << 16)&0xff0000 | (buf[3] << 8)&0xff00 | buf[4]&0xff;
            
            printf("recv refill %d %d\n", info.type, info.value);
        }
    }

    return info;
}

static void init_socket() {
    struct sockaddr_in serveraddr;
    
    FD_ZERO(&sock_fd_set);
    FD_ZERO(&sock_read_fds);
    
    server_sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (server_sock < 0)
    {
        perror("socket error : ");
        exit(0);
    }
    
    bzero(&serveraddr, sizeof(serveraddr));
    serveraddr.sin_family = AF_INET;
    serveraddr.sin_addr.s_addr = inet_addr("127.0.0.1");
    serveraddr.sin_port = htons(6500);
    
    if(bind(server_sock, (struct sockaddr *)&serveraddr, sizeof(serveraddr)) == -1)
    {
        perror("bind error: ");
        exit(0);
    }
    
    FD_SET(server_sock, &sock_fd_set);
    
    timeout.tv_sec = 0;
    timeout.tv_usec = 0;
}